package edu.uiuc.dsi.clocks;

/** A class implementing a simple Lamport Clock
 * 
 * @author Wade Fagen
 * @since 0.0.1.8
 */

public class LamportClock implements Comparable<LamportClock> 
{
	/**
	 * The current time of the clock. 
	 */
	private short iCurrentTime = 0;
	
	/**
	 * The default constructor, creating a <code>LamportClock</code> with a time of 0.
	 */
	public LamportClock() { }
	
	/**
	 * A secondary constructor, creating a <code>LamportClock</code> with a specified time. 
	 * @param time The initial time of the <code>LamportClock</code>.
	 */
	public LamportClock(short time) { iCurrentTime = time; }
	
	/**
	 * Increments the <code>LamportClock</code>'s clock. 
	 * @return The new time (after the tick) of the <code>LamportClock</code>. 
	 */
	public short tick()
	{
		return ++iCurrentTime;
	}
	
	/**
	 * 
	 * Increments the <code>LamportClock</code>'s clock with respect to a remote time. If the <code>remoteTime</code> is in the future,
	 * the <code>LamportClock</code> will be set to the <code>remoteTime</code> and tick. If the <code>remoteTime</code> is in the same time
	 * or in the past of the <code>LamportClock</code>, the <code>LamportClock</code> will simply tick.
	 * 
	 * @param remoteTime The time of a remote <code>LamportClock</code>.
	 * @return The new time (after the tick) of the <code>LamportClock</code>.
	 * @see #tick()
	 */
	
	public short tick(short remoteTime)
	{
		if (compareTimes(iCurrentTime, remoteTime) > 0)
			return this.tick();
		else
		{
			iCurrentTime = remoteTime;
			return this.tick();
		}
	}
	
	/**
	 * Retrieves the current time of the <code>LamportClock</code>.
	 * @return The current time of the <code>LamportClock</code>.
	 */
	public short getCurrentTime()
	{
		return iCurrentTime;
	}

	/**
	 * Compares two LamportClock times. Returns a value greater than zero if <code>time1</code> is ahead of <code>time2</code>, a value equal to zero
	 * if <code>time1</code> and <code>time2</code> are equal, and returns a value less than zero if <code>time1</code> is behind <code>time2</code>.  
	 * 
	 * @see compareTo(LamportClock)
	 * @param time1 The time of the first <code>LamportClock</code>.
	 * @param time2 The time of the second <code>LamportClock</code>.
	 * @return A positive value if <code>time1</code> is ahead of <code>time2</code>, a zero value if <code>time1</code> is equal to <code>time2</code>,
	 * and a negative value if <code>time1</code> is behind <code>time2</code>.
	 */
	private int compareTimes(short time1, short time2)
	{
		int intTime1 = (int)time1, intTime2 = (int)time2; 
		
		if (time1 < 0 && time2 >= 0)
			return intTime1 + (int)Short.MAX_VALUE - intTime2;
		else if (time1 >= 0 && time2 < 0)
			return intTime1 - (intTime2 + (int)Short.MAX_VALUE);
		else
			return intTime1 - intTime2;
	}

	@Override
	public int compareTo(LamportClock o)
	{
		return compareTimes(this.getCurrentTime(), o.getCurrentTime());
	}
	
	
}
